def test_import():

    import gen3


def test_indexclient_import():

    from gen3.indexclient.client import IndexClient


def test_auth_import():

    from gen3.auth import Gen3Auth


def test_submission_import():

    from gen3.submission import Gen3Submission


def test_file_import():

    from gen3.file import Gen3File
